@@ -1,9 +1,9 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "migemo" "1.9.2"
+(define-package "migemo" "20250616.309"
   "Japanese incremental search through dynamic pattern expansion."
-  '((cl-lib "0.5"))
+  '((emacs "25"))
   :url "https://github.com/emacs-jp/migemo"
-  :commit "f42832c8ac462ecbec9a16eb781194f876fba64a"
-  :revdesc "f42832c8ac46"
+  :commit "c0d84b4092ddade01110ba875559bfd454862ac2"
+  :revdesc "c0d84b4092dd"
   :authors '(("Satoru Takabayashi" . "satoru-t@is.aist-nara.ac.jp"))
   :maintainers '(("Satoru Takabayashi" . "satoru-t@is.aist-nara.ac.jp")))
